@@ -3,13 +3,14 @@ import math
 import logging
 from tqdm import tqdm
 from enum import Enum
+from copy import deepcopy
 from collections import OrderedDict
 from collections import defaultdict
 import numpy as np
 import torch
 from torch.utils.tensorboard import SummaryWriter
 from .state import RunnerState
-from pytorch_tools.utils.misc import listify
+import pytorch_tools.utils.misc as utils
 from pytorch_tools.utils.visualization import plot_confusion_matrix
 from pytorch_tools.utils.visualization import render_figure_to_tensor
 
@@ -65,7 +66,7 @@ class Callbacks(Callback):
 
     def __init__(self, callbacks):
         super().__init__()
-        self.callbacks = listify(callbacks)
+        self.callbacks = utils.listify(callbacks)
 
     def set_state(self, state):
         for callback in self.callbacks:
@@ -113,24 +114,29 @@ class Timer(Callback):
     def __init__(self):
         super().__init__()
         self.has_printed = False
+        self.timer = utils.TimeMeter()
 
     def on_batch_begin(self):
-        self.state.timer.batch_start()
+        self.timer.batch_start()
 
     def on_batch_end(self):
-        self.state.timer.batch_end()
+        self.timer.batch_end()
+
+    def on_loader_begin(self):
+        self.timer.reset()
 
     def on_loader_end(self):
         if not self.has_printed:
             self.has_printed = True
-            d_time = self.state.timer.data_time.avg_smooth
-            b_time = self.state.timer.batch_time.avg_smooth
+            d_time = self.timer.data_time.avg_smooth
+            b_time = self.timer.batch_time.avg_smooth
             print(f"\nTimeMeter profiling. Data time: {d_time:.2E}s. Model time: {b_time:.2E}s \n")
 
 
 class PhasesScheduler(Callback):
     """
     Scheduler that uses `phases` to process updates.
+    Supported `mode`'s are {`linear`, `cos`, `poly`}
 
     Args:
         phases (List[Dict]): phases
@@ -155,9 +161,9 @@ class PhasesScheduler(Callback):
         super(PhasesScheduler, self).__init__()
 
     def _format_phase(self, phase):
-        phase["ep"] = listify(phase["ep"])
-        phase["lr"] = listify(phase["lr"])
-        phase["mom"] = listify(phase.get("mom", None))  # optional
+        phase["ep"] = utils.listify(phase["ep"])
+        phase["lr"] = utils.listify(phase["lr"])
+        phase["mom"] = utils.listify(phase.get("mom", None))  # optional
         if len(phase["lr"]) == 2 or len(phase["mom"]) == 2:
             phase["mode"] = phase.get("mode", "linear")
             assert len(phase["ep"]) == 2, "Linear learning rates must contain end epoch"
@@ -170,6 +176,11 @@ class PhasesScheduler(Callback):
             return start + (end - start) * pct
         elif mode == "cos":
             return end + (start - end) / 2 * (math.cos(math.pi * pct) + 1)
+        elif mode == "poly":
+            gamma = (end / start) ** (1 / 100)
+            return start * gamma ** (pct * 100)
+        else:
+            raise ValueError(f"Mode: `{mode}` is not supported in PhasesScheduler")
 
     def _get_lr_mom(self, batch_curr):
         phase = self.phase
@@ -473,7 +484,8 @@ class ConsoleLogger(Callback):
 
 
 class FileLogger(Callback):
-    """Logs loss and metrics every epoch into file
+    """Logs loss and metrics every epoch into file.
+    If launched in distributed mode - reduces metrics before logging
     Args:
         log_dir (str): path where to store the logs
         logger (logging.Logger): external logger. Default None
@@ -488,6 +500,8 @@ class FileLogger(Callback):
         self.logger.info(f"Epoch {self.state.epoch_log} | lr {self.current_lr:.3f}")
 
     def on_epoch_end(self):
+        if utils.env_world_size() > 1:
+            self.reduce_metrics()
         loss, metrics = self.state.train_loss, self.state.train_metrics
         self.logger.info("Train " + self._format_meters(loss, metrics))
         if self.state.val_loss is not None:
@@ -513,6 +527,17 @@ class FileLogger(Callback):
     def _format_meters(loss, metrics):
         return f"loss: {loss.avg:.4f} | " + " | ".join(f"{m.name}: {m.avg:.4f}" for m in metrics)
 
+    def reduce_metrics(self):
+        # can't reduce AverageMeter so need to reduce every attribute separately
+        meters = self.state.train_metrics + [self.state.train_loss,]
+        meters = meters + self.state.metric_meters + [self.state.loss_meter,]
+        if self.state.val_loss is not None:
+            meters = meters + self.state.val_metrics + [self.state.val_loss,]
+        reduce_attributes = ["val", "avg", "avg_smooth", "sum", "count"]
+        for meter in meters:
+            for attr in reduce_attributes:
+                old_value = utils.to_tensor([getattr(meter, attr)]).float().cuda()
+                setattr(meter, attr, utils.reduce_tensor(old_value).cpu().numpy()[0])
 
 class Mixup(Callback):
     """Performs mixup on input. Only for classification.
@@ -684,4 +709,68 @@ class ResetOptimizer(Callback):
                 self.state.optimizer.state = defaultdict(dict)
                 
             if self.verbose:
-                print("Reseting optimizer")
\ No newline at end of file
+                print("Reseting optimizer")
+
+# docstring from https://github.com/rwightman/pytorch-image-models
+class ModelEma(Callback):
+    """ Model Exponential Moving Average
+    Keeps a moving average of everything in the model state_dict (parameters and buffers).
+    This is intended to allow functionality like
+    https://www.tensorflow.org/api_docs/python/tf/train/ExponentialMovingAverage
+
+    A smoothed version of the weights is necessary for some training schemes to perform well.
+    E.g. Google's hyper-params for training MNASNet, MobileNet-V3, EfficientNet, etc that use
+    RMSprop with a short 2.4-3 epoch decay period and slow LR decay rate of .96-.99 requires EMA
+    smoothing of weights to match results. 
+    
+    Current implementation follows TensorFlow and uses the following formula:
+    ema -= (1 - decay) * (ema - model)
+    This is mathematically equivalent to the classic formula below but inplace is faster
+    ema = decay * ema + (1 - decay) * model
+
+    NOTE: Pay attention to the decay constant you are using relative to your update count per epoch.
+    
+    NOTE: put this Callback AFTER Checkpoint saver! Otherwise you would validate EMA weights but save
+    model weights
+
+    NOTE: Need to be used in all process (not only master)! otherwise you would save not the best model bur some random
+
+    NOTE: Pass model to ModelEma after cuda() and AMP but before SyncBN and DDP wrapper
+
+    Args:
+        model (nn.Module): model after cuda and AMP
+        decay (float): decay for EMA for every step
+        decay_every (int): how oftern to really decay weights. Decaying every step produced a 
+            visible training slowdown. Real decay factor is adjusted to match every step update.
+    """
+    def __init__(self, model, decay=0.9999, decay_every=10):
+        super().__init__()
+        self.ema = deepcopy(model).eval()    
+        for p in self.ema.parameters():
+            p.requires_grad_(False)
+        self.model_copy = None
+        self.decay_factor = 1 - decay ** decay_every # simulate every step decay
+        self.decay_every = decay_every
+
+    def on_batch_end(self):
+        if not self.state.is_train or (self.state.step % self.decay_every != 0):
+            return
+
+        with torch.no_grad():
+            for (ema_v, m_v) in zip(self.ema.state_dict().values(), self.state.model.state_dict().values()):
+                if m_v.numel() == 1: # to prevent errors on `num_batches_tracked` in BN
+                    continue 
+                ema_v.sub_(ema_v.sub(m_v), alpha=self.decay_factor)
+
+    def on_loader_begin(self):
+        if self.state.is_train:
+           return
+        # validate on ema model
+        self.model_copy = self.state.model
+        self.state.model = self.ema
+        
+    def on_epoch_end(self):
+        if self.state.is_train:
+           return
+        # return model back
+        self.state.model = self.model_copy
\ No newline at end of file
