@@ -13,6 +13,7 @@ from .residual import TBottleneck
 from .residual import SEModule
 
 # from .residual import Transition, DenseLayer
+from .weight_standartization import conv_to_ws_conv
 
 from .activations import ACT_DICT
 from .activations import ACT_FUNC_DICT
@@ -21,8 +22,10 @@ from .activations import Mish, MishNaive, Swish, SwishNaive
 
 from .activated_batch_norm import ABN
 from .activated_group_norm import AGN
+from .activated_no_norm import NoNormAct
 from inplace_abn import InPlaceABN, InPlaceABNSync
 
+
 def bn_from_name(norm_name):
     norm_name = norm_name.lower()
     if norm_name == "abn":
@@ -35,5 +38,7 @@ def bn_from_name(norm_name):
         return partial(ABN, frozen=True)
     elif norm_name in ("agn", "groupnorm", "group_norm"):
         return AGN
+    elif norm_name in ("none",):
+        return NoNormAct
     else:
         raise ValueError(f"Normalization {norm_name} not supported")
